# ========================================================================
=head1 NAME

    DBIwrap.pm ---- a wrapper for select methods of all DBI versions

=head1 SAMPLE

    use DBIwrap;

    my $DSN = [ "dbi:SQLite2:dbname=database.sqlite","","" ];
    my $ATTR = { RaiseError => 1, AutoCommit => 1 };
    my $dbh = DBIwrap::delay->connect( @$DSN, $ATTR );

    my $where = $dbh->sql_where( { aa => 'abc' } );
    my $sql1 = "SELECT * FROM table WHERE $where ORDER BY aa;";

    $value = $dbh->select_1st_value( $sql1 );   # scalar selectrow_array
    $array = $dbh->select_1st_array( $sql1 );   # selectrow_arrayref
    $hash  = $dbh->select_1st_hash( $sql1 );    # selectrow_hashref
    $array = $dbh->select_all_value( $sql1 );   # selectcol_arrayref
    $array = $dbh->select_all_array( $sql1 );   # selectall_arrayref
    $array = $dbh->select_all_hash( $sql1 );    # selectall_hashref

    my( $ikey, $ival ) = DBIwrap::sql_insert( $dbh, $hash );
    my $sql2 = "INSERT INTO table ($ikey) VALUES ($ival);";
    $dbh->do( $sql2 );

    my $uset = DBIwrap::sql_update( $dbh, $hash );
    my $sql3 = "UPDATE table SET $uset WHERE $where;";
    $dbh->do( $sql3 );

=head1 COMPATIBILITY

    All methods in DBIwrap use only some methods and propaties following:

        $dbh->prepare( STATEMENT );
        $sth->execute( BIND_VALUES );
        $sth->fetchrow_arrayref();
        $sth->{NAME_lc};

    so DBIwrap is not require the current version of DBI.pm, 
    but is also available under older versions.

=cut
# ========================================================================
#   2001/06/16  KDB_Pg.pm module
#   2004/07/14  final release of KDB_DBI.pm module
#   2004/11/14  separate into DBIwrap.pm module
#   2004/11/30  older DBI v1.14 and DBD::Oracle supported, sql_where bug fix
# ========================================================================
    use strict;
    package DBIwrap;
    use DBI;
    use vars qw( $VERSION $DEBUG );
# ------------------------------------------------------------------------
    $VERSION = "0.01";
    @DBIwrap::ISA = qw( DBI );
    @DBIwrap::db::ISA = qw( DBI::db );
    @DBIwrap::dr::ISA = qw( DBI::dr );
    @DBIwrap::st::ISA = qw( DBI::st );
# ------------------------------------------------------------------------
sub connect_old_dbi {
    my $package = shift;
    my $dbh = DBI::connect( $package, @_ );
    bless $dbh, "$package\:\:db";
    $dbh;
}
# ------------------------------------------------------------------------
if ( $DBI::VERSION <= 1.14 ) {
    *connect = *connect_old_dbi;
}
# ========================================================================
#   DATABASE
# ========================================================================
    package DBIwrap::db;
    use vars qw( $DEBUG );
    $DEBUG = $DBIwrap::DEBUG;
# ------------------------------------------------------------------------
#   return array-ref of all column from all record
# ------------------------------------------------------------------------
sub select_all_array {
    my $dbh = shift or return;
    $DEBUG and print $DEBUG "[select_all_array] ",join(" ",@_),"\n";
    my $statement = shift;
    my $bind_values = \@_;
    my $sth = $dbh->prepare( $statement ) or return;
    $sth->execute( @$bind_values );
    my $array = [];
    while ( my $row = $sth->fetchrow_arrayref() ) {
        push( @$array, [ @$row ] );             # copy memory for safety
    }
    return undef unless scalar @$array;         # no results
    wantarray ? ( $array, $sth->{NAME_lc} ) : $array;
}
# ------------------------------------------------------------------------
#   return hash-ref of all column from all record
# ------------------------------------------------------------------------
sub select_all_hash {
    my $dbh = shift or return;
    $DEBUG and print $DEBUG "[select_all_hash] ",join(" ",@_),"\n";
    my $statement = shift;
    my $bind_values = \@_;
    my $sth = $dbh->prepare( $statement ) or return;
    $sth->execute( @$bind_values );
    my $cols = $sth->{NAME_lc};
    my $array = [];
    while ( my $row = $sth->fetchrow_arrayref() ) {
        my %hash;
        @hash{ @$cols } = @$row;                # marge two array into a hash
        push( @$array, \%hash );
    }
    return undef unless scalar @$array;         # no results
    wantarray ? ( $array, $cols ) : $array;
}
# ------------------------------------------------------------------------
#   return array-ref of first value from all record
# ------------------------------------------------------------------------
sub select_all_value {
    my $dbh = shift or return;
    $DEBUG and print $DEBUG "[select_all_value] ",join(" ",@_),"\n";
    my $statement = shift;
    my $bind_values = \@_;
    my $sth = $dbh->prepare( $statement ) or return;
    $sth->execute( @$bind_values );
    my $array = [];
    while ( my $row = $sth->fetchrow_arrayref() ) {
        push( @$array, $row->[0] );                 # copy first column only
    }
    return undef unless ref $array;                 # no results
    return undef unless scalar @$array;             # no results
    $array;
}
# ------------------------------------------------------------------------
#   return first record as array-ref
# ------------------------------------------------------------------------
sub select_1st_array {
    my $dbh = shift or return;
    $DEBUG and print $DEBUG "[select_1st_array] ",join(" ",@_),"\n";
    my $statement = shift;
    my $bind_values = \@_;
    my $sth = $dbh->prepare( $statement ) or return;
    $sth->execute( @$bind_values );
    my $row = $sth->fetchrow_arrayref() or return;
    return undef unless scalar @$row;               # no results
    wantarray ? ( $row, $sth->{NAME_lc} ) : $row;   # first row
}
# ------------------------------------------------------------------------
#   return first record as hash-ref
# ------------------------------------------------------------------------
sub select_1st_hash {
    my $dbh = shift or return;
    $DEBUG and print $DEBUG "[select_1st_hash] ",join(" ",@_),"\n";
    my $statement = shift;
    my $bind_values = \@_;
    my $sth = $dbh->prepare( $statement ) or return;
    $sth->execute( @$bind_values );
    my $cols = $sth->{NAME_lc};
    my $row = $sth->fetchrow_arrayref();
    my %hash;
    @hash{ @$cols } = @$row;                # marge two array into a hash
    wantarray ? ( \%hash, $cols ) : \%hash;
}
# ------------------------------------------------------------------------
#   return first column of first record
# ------------------------------------------------------------------------
sub select_1st_value { 
    my $dbh = shift or return;
    $DEBUG and print $DEBUG "[select_1st_value] ",join(" ",@_),"\n";
    my $statement = shift;
    my $bind_values = \@_;
    my $sth = $dbh->prepare( $statement ) or return;
    $sth->execute( @$bind_values );
    my $row = $sth->fetchrow_arrayref() or return;
    return undef unless scalar @$row;               # no results
    $row->[0];                                      # first column only
}
# ------------------------------------------------------------------------
#   generate where statement with right value quoted automatically
# ------------------------------------------------------------------------
sub sql_where {
    my $dbh = shift;
    my $arg = shift;
    my $join = shift || "AND";
    $DEBUG and print $DEBUG "[where] $dbh $arg $join\n";
    return ( $arg ne "" ) ? $arg : undef unless ref $arg;

    my $array = [];
    if ( ref $arg eq "HASH" ) {
        foreach my $left ( sort keys %$arg ) {
            my $right = $arg->{$left};
            next if ( $left eq "" );
            if ( $left =~ /^and$/i ) {
                my $child = $dbh->sql_where( $right, "AND" );
                push( @$array, "( $child )" ) if defined $child;
            } elsif ( $left =~ /^or$/i ) {
                my $child = $dbh->sql_where( $right, "OR" );
                push( @$array, "( $child )" ) if defined $child;
            } elsif ( $left =~ /^not$/i ) {
                my $child = $dbh->sql_where( $right, "AND" );
                push( @$array, "NOT ( $child )" ) if defined $child;
            } elsif ( ref $right ) {                  # IN
                next unless scalar @{$right};           # NULL
                my $quoted = join( ", "=> map { $dbh->quote($_) } @$right );
                push( @$array, "$left IN ( $quoted )" );
            } else {
                push( @$array, "$left = ".$dbh->quote($right) );
            }
        }
    } elsif ( ref $arg eq "ARRAY" ) {
        @$array = map {"($_)"} map { $dbh->sql_where( $_, "AND" ) } @$arg;
    }
    return undef unless scalar @$array;             # no arguments
    return join( " $join ", @$array );              # join by "and" or "or"
}
# ------------------------------------------------------------------------
sub sql_insert {
    my $dbh = shift;
    my $hash = shift;
    my $klist = shift || [ sort keys %$hash ];
    return unless ref $hash;
    return unless scalar @$klist;
    my $kstr = join( ", ", @$klist );
    return $kstr unless wantarray;
    my $vlist = [ map { $dbh->quote($hash->{$_}) } @$klist ];
    return unless scalar @$vlist;
    my $vstr = join( ", ", @$vlist );
    return ( $kstr, $vstr );                # keys and values
}
# ------------------------------------------------------------------------
sub sql_update {
    my $dbh = shift;
    my $hash = shift;
    my $klist = shift || [ sort keys %$hash ];
    return unless ref $hash;
    return unless scalar @$klist;
    my $vlist = [ map {"$_ = ".$dbh->quote($hash->{$_})} @$klist ];
    return unless scalar @$vlist;
    join( ", ", @$vlist );
}
# ========================================================================
    package DBIwrap::delay;
    use vars qw( $AUTOLOAD $DEBUG );
    $DEBUG = $DBIwrap::DEBUG;
# ------------------------------------------------------------------------
sub connect {
    my $package = shift;
    my $obj = [ @_ ];                           # copy dsn
    bless $obj, $package;
    $obj;
}
# ------------------------------------------------------------------------
sub AUTOLOAD {
    my $ref = \$_[0];                           # pointer
    my $obj = shift;                            # DBIdelay object
    my $arg = \@_;                              # method arguments
    return unless ref $obj;                     # something wrong?
    require DBI;
    my $dbh = DBIwrap->connect( @$obj );        # make connect now
    return unless ref $dbh;                     # connection failed
    $$ref = $dbh;                               # over write self
    my $call = ( $AUTOLOAD =~ /([^:]+)$/ )[0];  # called method name
    return $dbh->$call( @$arg );                # call original method
}
# ------------------------------------------------------------------------
sub DESTROY {
    undef;                                      # nothing proceeded
}
# ========================================================================
;1;
# ========================================================================
